/* --------------------------------------------------------------------------

	(c) 2004-2007 U.S. Robotics Corporation

	-------------------------------------------------------------------------- */

/* --------------------------------------------------------------------------

	Define the information for a menu bar.
	Each menu tab has an associated name and filename.
	The name is translated, but not the filename.

	The menu bar object contains an array of menu tabs.

	The calling page should call the appropriate members to construct a menubar.
	Example:
		initMenu(false);

		var bar = new MenuBar();

		bar.addTab("Status",		"status_tabbed.htm");
		bar.addTab("Log",			"log.htm");
		bar.addTab("Internet",	"wan.htm");
		bar.addTab("Security",	"security.htm");
		bar.addTab("Firewall",	"firewall.htm");
		bar.addTab("Wireless",	"wireless.htm");
		bar.addTab("LAN",			"lan.htm");
		bar.addTab("Device",		"device.htm");

		bar.write();

*/

function MenuBar()
{
	this._menus = new Array();


	this.addTab = function(strName, strFilename)
	{
		var tab = new MenuTab(strName, strFilename);
		this._menus.push(tab);
		return tab;
	}

	/*
		This writes out the menu bar and its dropdown menus.
	*/
	this.write = function()
	{
		/*
			If the page is right-to-left, reverse the order of the menu's members.

			The isPageRTL() function is defined in the usr_rtl.js file.
			If it's not available, assume left-to-right.
		*/
		if ((typeof(isPageRTL) != "undefined") && isPageRTL())
			this._menus.reverse();

		document.write("<div id=\"idMenu\">");
		for (var i = 0; i < this._menus.length; ++i)
			this._menus[i].write(this._iWidth);
		document.write("<\/div>");
	}
}


/* --------------------------------------------------------------------------

	Define the information for a single menu tab.
	Each tab has an associated filename and title.
	The title is translated, but not the filename.

*/

function MenuTab(strName, strFilename)
{
	this._strName = strName;
	this._strFilename = strFilename;


	this.write = function()
	{
		// We don't output a newline after each because we use CSS to configure the gutter between tabs.
		// If we output a newline (or space), the rendering engine would add gutter.
		if (getPageFilename() == this._strFilename)
			document.write("<span class=\"clsMenuSelected\">" + this._strName + "<\/span>");
		else
			document.write("<a class=\"clsMenu\" href=\"" + this._strFilename + "\">" + this._strName + "<\/a>");
	}
}


/*
	This function returns the filename of the current page.
	It looks for both slash and backward slash.
*/
function getPageFilename()
{
	var strFilename = document.URL;
	var ixSeparator = strFilename.lastIndexOf("/");
	var ixSeparator2 = strFilename.lastIndexOf("\\");
	if (ixSeparator2 > ixSeparator)
		ixSeparator = ixSeparator2;
	if (ixSeparator != -1)
		strFilename = strFilename.substr(ixSeparator + 1);
	return strFilename;
}
