/*
 * Copyright 2004, U.S. Robotics Corporation
 * All Rights Reserved.
 * 
 * THIS SOFTWARE IS OFFERED "AS IS", AND U.S. Robotics GRANTS NO WARRANTIES OF
 * ANY KIND, EXPRESS OR IMPLIED, BY STATUTE, COMMUNICATION OR OTHERWISE. U.S.
 * Robotics SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A SPECIFIC PURPOSE OR NONINFRINGEMENT CONCERNING THIS SOFTWARE.
 *
 * $Id: usr-hdr.c,v 1.1 2007/02/14 16:56:01 dtutuianu Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <trxhdr.h>
#include <usrhdr.h>
#include "crc.h"

#define USR_DEBUG

#ifdef USR_DEBUG
#	define dprintf(fmt,args...) printf(fmt, ## args); 
#else
#	define dprintf(args...) 
#endif

#define EPI_LABEL "EPI_VERSION"
#define USR_MAGIC 0x30525355	//	"USR0"
#define uint32 unsigned long
#define OneMbytes 1048576L

char *fetchvalue(char *p, int l)
{
    static char epi_value[50];
    int i;
    char *tp;
    
    tp = p + strlen(EPI_LABEL);
    if ((*tp != ' ') && (*tp != '\t')) return 0;

    while ((*tp != 0x0a) && ((*tp == ' ') || (*tp == '\t')) && (l > 0))
    {
	l--; *tp++;
    }
    if (l)
    {
	i = 0;
	while (*tp != 0x0a)
	    epi_value[i++] = *tp++;
	epi_value[i] = 0;
	dprintf("epi_value=%s\n",epi_value);
    } else return 0;
    return epi_value;
}


int getversion(char *path, struct usr_header *hdr)
{
    FILE *f;
    char tmp[256], *p, *v;
    int ret = 0;
    int M,m,rc,inc,b;
    struct timeval tv;

    if ((!path) || (!hdr)) return 0;
	
    f = fopen( path, "r" );
    if (f)
    {
        while(fgets(tmp, sizeof(tmp), f))
        {
	    if ((p = strstr(tmp, EPI_LABEL)))
	    {
		if ((v = fetchvalue(p, strlen(tmp) - (p-tmp))))
		{
		    sscanf(v, "%d,%d,%d,%d,%d",&M,&m,&rc,&inc,&b);
		    hdr->magic = USR_MAGIC;
		    hdr->version =  (M & 0x000000ff) |
				    ((m & 0x000000ff) << 8) |
				    ((rc & 0x000000ff) << 16) |
				    ((inc & 0x000000f0) << 24) |
				    ((b & 0x0000000f) << 24);
		    hdr->compatibility_id = COMPAT_ID;
		    hdr->hardware_revision = HARDWARE_REV;
		   // gettimeofday(&tv, NULL);
		   // hdr->time = tv.tv_sec;
		    ret = 1;
		    break;
		}
	    }
	}
	fclose(f);
    }
    return ret;
}

int createfile(char *path_trx,char *path_usr, struct usr_header *hdr)
{
    int ret = 0;
    FILE *f_trx, *f_usr;
    char *s;
    struct trx_header trx;
    unsigned long crc;
    int i,count,trx_mod,trx_div,off,start;
    // struct tm *t = 0;
    
    f_trx = fopen( path_trx, "r" );
    if (f_trx)
    {
        f_usr = fopen( path_usr, "w" );
	if (f_usr)
	{
	    count = fread(&trx, 1, sizeof(struct trx_header), f_trx);
    	    if (count >= sizeof(struct trx_header)) 
	    {
		if (trx.magic == TRX_MAGIC &&
		    trx.len < TRX_MAX_LEN &&
		    trx.len >= sizeof(struct trx_header)) 
		{
		    dprintf("trx.flag_version=%d,%d\n",
			((trx.flag_version >> 16) & 0xff),
			((trx.flag_version >> 24) & 0xff));

		    if ((s = malloc(OneMbytes))) 
		    {
    			fwrite(hdr, 1, sizeof(struct usr_header) , f_usr);
    			start = count = off = sizeof(struct trx_header);
			trx_mod = trx.len % OneMbytes;
			trx_div = trx.len / OneMbytes;
			// Calculate CRC over header 
			crc = crc32((uint8 *) &trx,sizeof(struct trx_header),CRC32_INIT_VALUE);
			for (i = 0; i < trx_div; i++)
			{
			    if (start)
			    {
				memcpy(s, &trx, sizeof(struct trx_header));
				start = 0;
			    } else {
    				count = off = 0;
			    }
        		    count += fread(&s[off], 1, OneMbytes - off , f_trx);
			    if (count < OneMbytes)
			    {
	    			dprintf("trx file was truncated (actual %ld expect %ld)\n", 
				    count - off, OneMbytes - off);
				goto fail;
			    }
			    crc = crc32(&s[off], count - off, crc);
    			    fwrite(s, 1, OneMbytes, f_usr);
			    
			}
    			count = fread(s, 1, trx_mod, f_trx);
    			crc = crc32(s, count, crc);
			fwrite(s, 1, trx_mod, f_usr);
			hdr->len = trx.len;
			hdr->crc32 = crc;
			fseek(f_usr, 0L, SEEK_SET);
    			fwrite(hdr, 1, sizeof(struct usr_header) , f_usr);
			
			dprintf("hdr.magic=%x\n",hdr->magic);
			dprintf("hdr.len=%d\n",hdr->len);
			dprintf("hdr.crc32=%x\n",hdr->crc32);
			dprintf("hdr.version=%x (%d, %d, %d, %d, %d)\n",
			    hdr->version, 
			    (hdr->version & 0xff), (hdr->version >> 8 ) & 0xff,
			    (hdr->version >> 16) & 0xff, (hdr->version >> 24) & 0xf0, (hdr->version >> 24) & 0x0f);
			dprintf("hdr.compatibitlity_id=%d\n",hdr->compatibility_id);
			dprintf("hdr.hardware_revision=%x\n",hdr->hardware_revision);
			/*t = gmtime(&hdr->time);
			if (t)
			{
			    dprintf("hdr.date=%d-%#02d-%d %#02d:%#02d:%#02d\n",
				t->tm_mday, t->tm_mon+1, t->tm_year+1900,
				t->tm_hour, t->tm_min,t->tm_sec);
			}*/
		    }
		    
	    	    ret = 1;
fail:				        
		    free(s);
		    fclose(f_trx);
		}
	    }
	    fclose (f_usr);
	}
    }
    
    return ret;
}



void usage()
{

    printf("\n");
    printf("*** U.S. Robotics Header Generator V 1.00 ***\n");
    printf("*** Copyright (c) 2004, U.S. Robotics     ***\n");
    printf("*** Alrights reserved..                   ***\n");
    printf("\n");
    printf("Usage: usr-hdr <location of epivers.h> <trx file> <output new firmware file>\n");
    printf("\n");
    printf("Ex: usr-hdr src/include/epivers.h images/linux.trx images/linux.usr\n");
    
}


int main( int argc, char *argv[] )
{
    int c;
    struct usr_header hdr;
    int ret = 1;

    for (c = 0; c < argc ; c++)
    {
	dprintf("arg%d = %s\n", c, argv[c]);
    }
    
            
    if (argc >= 3)
    {
	memset(&hdr,0, sizeof(struct usr_header));
	if (!getversion(argv[1], &hdr)) return -1;
	if (!createfile(argv[2], argv[3], &hdr)) return -2;
	ret = 0;
//        ret = 1;
    } else usage();

    return ret;
}
